with QuadDobl_Complex_Poly_Ring;
with Generic_Vectors;

package QuadDobl_Complex_Poly_Vectors is
  new Generic_Vectors(QuadDobl_Complex_Poly_Ring);

-- DESCRIPTION :
--   Defines vectors over the ring of quad double complex polynomials.
