with Generic_Polynomials;
with Octo_Double_Ring;

package Octo_Double_Polynomials is 
  new Generic_Polynomials(Octo_Double_Ring);

-- DESCRIPTION :
--   Defines polynomials over the ring of octo double numbers.
