package body Multiplicity_Sampling_Machine is

-- INTERNAL DATA STRUCTURES :

-- TARGET OPERATIONS :

  procedure Initialize ( sp : in Standard_Complex_Poly_Systems.Poly_Sys;
                         mp : in Multprec_Complex_Poly_Systems.Poly_Sys;
                         dim,ind : in natural ) is

  begin
    null;
  end Initialize;

  procedure Clear is
  begin
    null;
  end Clear;

end Multiplicity_Sampling_Machine;
