with QuadDobl_Complex_Ring;
with Generic_Vectors;

package QuadDobl_Complex_Vectors is new Generic_Vectors(QuadDobl_Complex_Ring);

-- DESCRIPTION :
--   Defines vectors over the ring of quad double complex numbers.
