package Test_Natural_Matrices is

-- DESCRIPTION :
--   Tests the matrix packages of standard and multi-precision naturals.

  procedure Test_Standard_io;

  -- DESCRIPTION :
  --   Prompts for the number of rows and columns and then
  --   for the entries of a matrix of 32-bit natural numbers.
  --   Writes the matrix to screen.

  procedure Test_Multprec_io;

  -- DESCRIPTION :
  --   Prompts for the number of rows and columns and then
  --   for the entries of a matrix of multiprecision natural numbers.
  --   Writes the matrix to screen.

  procedure Main;

  -- DESCRIPTION :
  --   Displays a menu and prompts for a test.

end Test_Natural_Matrices;
