package Test_QuadDobl_Matrices is

-- DESCRIPTION :
--   Tests the quad double matrix packages.

  procedure Test_Quad_Double_Matrices;

  -- DESCRIPTION :
  --   Prompts the user for dimensions and writes then
  --   a randomly generated matrix.

  procedure Test_Quad_Double_Complex_Matrices;

  -- DESCRIPTION :
  --   Prompts the user for number of rows and columns
  --   and then shows a matrix of random complex quad doubles.

  procedure Main;

  -- DESCRIPTION :
  --   Displays a menu and prompts for a test.

end Test_QuadDobl_Matrices;
