package Test_HexaDobl_Vectors is

-- DESCRIPTION :
--   Tests vectors in hexa double precision.

  procedure Test_Hexa_Double_Vectors;

  -- DESCRIPTION :
  --   Prompts the user for a dimension and writes then
  --   a randomly generated complex vector in hexa double precision.

  procedure Test_Hexa_Double_Complex_Vectors;

  -- DESCRIPTION :
  --   Prompts for a dimension and generates random complex vectors
  --   in hexa double precision.

  procedure Main;

  -- DESCRIPTION :
  --   Displays a menu and prompts for a test.

end Test_HexaDobl_Vectors;
