with OctoDobl_Complex_Series_Ring;
with Generic_Vectors;

package OctoDobl_Complex_Series_Vectors is 
  new Generic_Vectors(OctoDobl_Complex_Series_Ring);

-- DESCRIPTION :
--   Defines vectors over the ring of dense series
--   with octo double complex coefficients.
