with Test_PentDobl_Vector_Norms;

procedure ts_pdnorm is

-- DESCRIPTION :
--   Calls the main interactive test on vector norms
--   in penta double precision.

begin
  Test_PentDobl_Vector_Norms.Main;
end ts_pdnorm;
