with Test_TripDobl_Polynomials;

procedure ts_tdpol is

-- DESCRIPTION :
--   Calls the main test on polynomials with triple double coefficients.

begin
  Test_TripDobl_Polynomials.Main;
end ts_tdpol;
