with Generic_Polynomials;
with OctoDobl_Complex_Ring;

package OctoDobl_Complex_Polynomials is 
  new Generic_Polynomials(OctoDobl_Complex_Ring);

-- DESCRIPTION :
--   Defines polynomials over the ring of octo double complex numbers.
