with text_io;                            use text_io;
with Bye_Bye_Message;

procedure ts_byemess is

-- DESCRIPTION :
--   Prints the message to say bye bye at the end of the program.

begin
  put_line(Bye_Bye_Message);
end ts_byemess;
