with Test_TripDobl_Functions;

procedure ts_tdfun is

-- DESCRIPTION :
--   Calls the main interactive test on functions in triple double precision.

begin
  Test_TripDobl_Functions.Main;
end ts_tdfun;
