package Test_Maple_Solutions is

-- DESCRIPTION :
--   Test on the output of solution lists in Maple format.

  procedure Standard_PHCpack_to_Maple;

  -- DESCRIPTION :
  --   Converts solutions in the format of PHCpack into Maple.

  procedure Multprec_PHCpack_to_Maple;

  -- DESCRIPTION :
  --   Converts solutions in the format of PHCpack into Maple.

  procedure Standard_Maple_to_PHCpack;

  -- DESCRIPTION :
  --   Converts solutions in Maple to the format of PHCpack. 

  procedure Multprec_Maple_to_PHCpack;

  -- DESCRIPTION :
  --   Converts solutions in Maple to the format of PHCpack. 

  procedure Main;

  -- DESCRIPTION :
  --    Displays a menu and prompts for a format conversion choice.

end Test_Maple_Solutions;
