with PentDobl_Complex_Ring;
with Generic_Vectors;

package PentDobl_Complex_Vectors is new Generic_Vectors(PentDobl_Complex_Ring);

-- DESCRIPTION :
--   Defines vectors over the ring of penta double complex numbers.
