with Test_PentDobl_Polynomials;

procedure ts_pdpol is

-- DESCRIPTION :
--   Calls the main test on polynomials with penta double coefficients.

begin
  Test_PentDobl_Polynomials.Main;
end ts_pdpol;
