with QuadDobl_Complex_Series_Ring;
with Generic_Vectors;

package QuadDobl_Complex_Series_Vectors is 
  new Generic_Vectors(QuadDobl_Complex_Series_Ring);

-- DESCRIPTION :
--   Defines vectors over the ring of quaddobl dense series.
