with Generic_Polynomials;
with Triple_Double_Ring;

package Triple_Double_Polynomials is 
  new Generic_Polynomials(Triple_Double_Ring);

-- DESCRIPTION :
--   Defines polynomials over the ring of triple double numbers.
