with Standard_Complex_Series_Ring;
with Generic_Vectors;

package Standard_Complex_Series_Vectors is 
  new Generic_Vectors(Standard_Complex_Series_Ring);

-- DESCRIPTION :
--   Defines vectors over the ring of standard dense series.
