package Test_OctoDobl_Vectors is

-- DESCRIPTION :
--   Tests vectors in octo double precision.

  procedure Test_Octo_Double_Vectors;

  -- DESCRIPTION :
  --   Prompts the user for a dimension and writes then
  --   a randomly generated complex vector in octo double precision.

  procedure Test_Octo_Double_Complex_Vectors;

  -- DESCRIPTION :
  --   Prompts for a dimension and generates random complex vectors
  --   in octo double precision.

  procedure Main;

  -- DESCRIPTION :
  --   Displays a menu and prompts for a test.

end Test_OctoDobl_Vectors;
