with Test_Homotopy_Definitions;

procedure ts_induce is

-- DESCRIPTION :
--   Allows to interactively test the setup and the running of the
--   Littlewood-Richardson homotopies.

begin
  Test_Homotopy_Definitions.Main;
end ts_induce;
