with Generic_Polynomials;
with Standard_Complex_Ring;

package Standard_Complex_Polynomials is 
  new Generic_Polynomials(Standard_Complex_Ring);

-- DESCRIPTION :
--   Defines the polynomials over the ring of standard complex numbers.
