package Test_Rho_Algorithm is

-- DESCRIPTION :
--   Reproduces the results of the 1955 paper of Wynn
--   on the rho algorithm, using the example of the paper.

  procedure Main;

  -- DESCRIPTION :
  --   Prompts to select a test.

end Test_Rho_Algorithm;
