// The file random_vectors.h specifies functions to generate random numbers.

#ifndef __random_numbers_h__
#define __random_numbers_h__

double random_double ( void );
// Returns a random double in [-1,+1].

double random_angle ( void );
// Returns a random double in [0, 2*pi].

#endif
