with DoblDobl_Dense_Series_Ring;
with DoblDobl_Dense_Series_Vectors;
with Generic_Matrices;

package DoblDobl_Dense_Series_Matrices is 
  new Generic_Matrices(DoblDobl_Dense_Series_Ring,
                       DoblDobl_Dense_Series_Vectors);

-- DESCRIPTION :
--   Defines matrices over the ring of double double dense series.
