with Test_HexaDobl_Linear_Solvers;

procedure ts_hdlu is

-- DESCRIPTION :
--   Calls the main interactive solver of linear systems
--   in hexa double precision.

begin
  Test_HexaDobl_Linear_Solvers.Main;
end ts_hdlu;
