with Test_OctoDobl_Singular_Values;

procedure ts_odsvd is

-- DESCRIPTION :
--   Calls the main interactive test on the Singular Value Decomposition
--   in octo double precision.

begin
  Test_OctoDobl_Singular_Values.Main;
end ts_odsvd;
