with Standard_Integer_Numbers;           use Standard_Integer_Numbers;

package Main_ADE_Trackers is

-- DESCRIPTION :
--   ADE stands for Algorithmic Differentiation and Evaluation.

  procedure Main ( vrblvl : in integer32 := 0 );

  -- DESCRIPTION :
  --   This is the main interface to the path trackers
  --   with algorithmic differentiation methods.
  --   The value of the verbose level is in vrblvl.

end Main_ADE_Trackers;
