with Test_PentDobl_Linear_Solvers;

procedure ts_pdlu is

-- DESCRIPTION :
--   Calls the main interactive solver of linear systems
--   in penta double precision.

begin
  Test_PentDobl_Linear_Solvers.Main;
end ts_pdlu;
