with Generic_Bracket_Polynomials;
with QuadDobl_Complex_Ring;

package QuadDobl_Bracket_Polynomials is 
  new Generic_Bracket_Polynomials(QuadDobl_Complex_Ring);

-- DESCRIPTION :
--   Bracket polynomials over the ring of quad double complex numbers.
