with Generic_Polynomials;
with TripDobl_Complex_Ring;

package TripDobl_Complex_Polynomials is 
  new Generic_Polynomials(TripDobl_Complex_Ring);

-- DESCRIPTION :
--   Defines polynomials over the ring of triple double complex numbers.
