with Standard_Natural_Ring;
with Generic_Vectors;

package Standard_Natural_Vectors is new Generic_Vectors(Standard_Natural_Ring);

-- DESCRIPTION :
--   Defines vectors over the ring of standard natural numbers.
