with Generic_Polynomials;
with DecaDobl_Complex_Ring;

package DecaDobl_Complex_Polynomials is 
  new Generic_Polynomials(DecaDobl_Complex_Ring);

-- DESCRIPTION :
--   Defines polynomials over the ring of deca double complex numbers.
