with Test_HexaDobl_QRLS_Solvers;

procedure ts_hdqrd is

-- DESCRIPTION :
--   Calls the main interactive test on the QR Decomposition and
--   the least squares solver in hexa double precision.

begin
  Test_HexaDobl_QRLS_Solvers.Main;
end ts_hdqrd;
