package Test_DoblDobl_Functions is

-- DESCRIPTION :
--   Tests mathematical functions in double double precision.

  procedure Test_SINCOS;

  -- DESCRIPTION :
  --   Verifies if the square of sine and cosine on a random double double
  --   or a user given number sum up to one.

  procedure Test_SQRT;

  -- DESCRIPTION :
  --   Compares the square of the square root of a random or given number.

  procedure Main;

  -- DESCRIPTION :
  --   Displays a menu and prompts for a test.
  
end Test_DoblDobl_Functions;
