with Standard_Integer_Numbers;          use Standard_Integer_Numbers;

function Number_of_Cores return integer32;

-- DESCRIPTION :
--   Returns the number of available cores,
--   or -1 if something went wrong.
