procedure postpoco;

-- DESCRIPTION :
--   Scans the output file of poco for the computed directions,
--   residuals and estimated multiplicities of the solutions.
--   This is an intermediate stage in the polyhedral end game.
