with Triple_Double_Ring;
with Generic_Vectors;

package Triple_Double_Vectors is
  new Generic_Vectors(Triple_Double_Ring);

-- DESCRIPTION :
--   Defines vectors over the ring of triple double numbers.
