with Test_mtShift_Convolutions;

procedure ts_mtshiftcnv is

-- DESCRIPTION :
--   Calls the main test on the multitasked code
--   to shift coefficients of circuits.

begin
  Test_mtShift_Convolutions.Main;
end ts_mtshiftcnv;
