with DoblDobl_Complex_Ring;
with Generic_Vectors;

package DoblDobl_Complex_Vectors is new Generic_Vectors(DoblDobl_Complex_Ring);

-- DESCRIPTION :
--   Defines vectors over the ring of double double complex numbers.
