with DecaDobl_Complex_Ring;
with Generic_Vectors;

package DecaDobl_Complex_Vectors is new Generic_Vectors(DecaDobl_Complex_Ring);

-- DESCRIPTION :
--   Defines vectors over the ring of deca double complex numbers.
