/* The class complex defines a complex type for GPU computations.
   The template parameter value T specifies precision level */

#ifndef __COMPLEX_H__
#define __COMPLEX_H__

#define cd  0
#define cdd 1
#define cqd 2

#include "complex_gd.h"
#include "complex_gdd.h"
#include "complex_gqd.h"

#endif /*__COMPLEX_H__*/
