with Generic_Polynomials;
with PentDobl_Complex_Ring;

package PentDobl_Complex_Polynomials is 
  new Generic_Polynomials(PentDobl_Complex_Ring);

-- DESCRIPTION :
--   Defines polynomials over the ring of penta double complex numbers.
