with Test_Dictionary_Solutions;

procedure ts_dicsols_io is

-- DESCRIPTION :
--   Test on the output of solution lists in dictionary format.

begin
  Test_Dictionary_Solutions.Main;
end ts_dicsols_io;
