with Test_Complex_Numbers;

procedure ts_cmpnum is

-- DESCRIPTION :
--   Calls the main interactive test on complex number arithmetic,
--   in standard double and multiple precision.

begin
  Test_Complex_Numbers.Main;
end ts_cmpnum;
