with Test_Polar_Exponentiation;

procedure ts_cmpexp is

-- DESCRIPTION :
--   Calls the main interactive test on the exponentiation of complex numbers,
--   via the polar representation.

begin
  Test_Polar_Exponentiation.Main;
end ts_cmpexp;
