with Standard_Natural_Numbers;          use Standard_Natural_Numbers;

function Pieri_Root_Count ( m,p,q : natural32 ) return natural32;

-- DESCRIPTION :
--   Returns the number of degree q maps that produce p-planes meeting
--   m*p general m-planes in (m+p)-space.
