package Test_System_and_Solutions_io is

-- DESCRIPTION :
--   Tests the reading of a system and its solutions.

  procedure Double_Test_Get;
  procedure DoblDobl_Test_Get;
  procedure TripDobl_Test_Get;
  procedure QuadDobl_Test_Get;
  procedure PentDobl_Test_Get;
  procedure OctoDobl_Test_Get;
  procedure DecaDobl_Test_Get;
  procedure HexaDobl_Test_Get;

  -- DESCRIPTION :
  --   Prompts for a system with solutions 
  --   in double, double double, quad double, penta double,
  --   octo double, deca double, and hexa double precision.

  procedure Main;

  -- DESCRIPTION :
  --   Displays a menu and then runs the selected test.

end Test_System_and_Solutions_io;
