package Test_HexaDobl_Polynomials is

-- DESCRIPTION :
--   Tests polynomials with hexa double coefficients.

  procedure Test_Polynomial_Convertor;

  -- DESCRIPTION :
  --   Tests the conversions of double complex coefficient polynomials
  --   with hexa double complex coefficient polynomials.

  procedure Test_Laurent_Polynomial_Convertor;

  -- DESCRIPTION :
  --   Tests the conversions of double complex coefficient Laurent
  --   polynomials with hexa double complex coefficient polynomials.

  procedure Main;

  -- DESCRIPTION :
  --   Displays a menu and prompts for a test.

end Test_HexaDobl_Polynomials;
