with Test_Schubert_Conditions;

procedure ts_flagcond is

-- DESCRIPTION :
--   Main test on the symbolic and numeric Schubert intersection conditions.

begin
  Test_Schubert_Conditions.Main;
end ts_flagcond;
