with Test_mtHessian_Convolutions;

procedure ts_mthessian is

-- DESCRIPTION :
--   Calls the main test on the computation of Hessian for
--   systems of convolution circuits.

begin
  Test_mtHessian_Convolutions.Main;
end ts_mthessian;
