with Test_Double_Lseries_Polynomials;

procedure ts_lserpol is

-- DESCRIPTION :
--   Calls the main test on the evaluation and differention of polynomials
--   at Laurent series, in double precision.

begin
  Test_Double_Lseries_Polynomials.Main;
end ts_lserpol;
