with Test_HexaDobl_Singular_Values;

procedure ts_hdsvd is

-- DESCRIPTION :
--   Calls the main interactive test on the Singular Value Decomposition
--   in hexa double precision.

begin
  Test_HexaDobl_Singular_Values.Main;
end ts_hdsvd;
