with Multprec_Complex_Poly_Ring;
with Generic_Vectors;

package Multprec_Complex_Poly_Vectors is
  new Generic_Vectors(Multprec_Complex_Poly_Ring);

-- DESCRIPTION :
--   Defines vectors over the ring of multiprecision complex polynomials.
