with TripDobl_Complex_Ring;
with Generic_Vectors;

package TripDobl_Complex_Vectors is new Generic_Vectors(TripDobl_Complex_Ring);

-- DESCRIPTION :
--   Defines vectors over the ring of triple double complex numbers.
