with Test_HexaDobl_Functions;

procedure ts_hdfun is

-- DESCRIPTION :
--   Calls the main interactive test on functions in hexa double precision.

begin
  Test_HexaDobl_Functions.Main;
end ts_hdfun;
