with Multprec_Integer_Ring;
with Generic_Vectors;

package Multprec_Integer_Vectors is new Generic_Vectors(Multprec_Integer_Ring);

-- DESCRIPTION :
--   Defines vectors over the ring of multi-precision integer numbers.
