/* Configuration for phcpack generated by Alire */
#ifndef PHCPACK_CONFIG_H
#define PHCPACK_CONFIG_H

#define CRATE_VERSION "2.4.88"
#define CRATE_NAME "phcpack"

#define ALIRE_HOST_OS "windows"
// #define ALIRE_HOST_OS "linux"

#define ALIRE_HOST_ARCH "x86_64"

#define ALIRE_HOST_DISTRO "msys2"

#define BUILD_PROFILE_RELEASE 1
#define BUILD_PROFILE_VALIDATION 2
#define BUILD_PROFILE_DEVELOPMENT 3

#define BUILD_PROFILE  3

#endif
