/*
 * DefineType_Device.h
 *
 *  Created on: Feb 1, 2015
 *      Author: yxc
 */

#ifndef DEFINE_TYPE_QD_DEVICE_H_
#define DEFINE_TYPE_QD_DEVICE_H_

#include "../Complex/complex.h"

typedef gqd_real T;

#define GT gqd_complex

#endif /* DEFINE_TYPE_QD_DEVICE_H_ */
