package Test_TripDobl_Functions is

-- DESCRIPTION :
--   Tests functions in triple double precision.

  procedure Test_SQRT;

  -- DESCRIPTION :
  --   Compares the square of the square root of a random or given number.

  procedure Test_Radius;

  -- DESCRIPTION :
  --   Verifies the radius with Pythagorean triple.

  procedure Test_SINCOS;

  -- DESCRIPTION :
  --   Verifies if the square of sine and cosine on a random triple double
  --   or a user given number sum up to one.

  procedure Main;

  -- DESCRIPTION :
  --   Displays a menu and prompts for a test.

end Test_TripDobl_Functions;
