with Test_HexaDobl_Vector_Norms;

procedure ts_hdnorm is

-- DESCRIPTION :
--   Calls the main interactive test on vector norms
--   in hexa double precision.

begin
  Test_HexaDobl_Vector_Norms.Main;
end ts_hdnorm;
