procedure phctop;

-- DESCRIPTION :
--   This procedure processes the command line arguments
--   and calls the proper procedures corresponding to the options.
--   This is the top level procedure for the executable phc.
