package Test_DoblDobl_Matrices is

-- DESCRIPTION :
--   Tests matrices in double double precision.

  procedure Test_Double_Double_Matrices;

  -- DESCRIPTION :
  --   Prompts the user for dimensions and writes then
  --   a randomly generated matrix.

  procedure Test_Double_Double_Complex_Matrices;

  -- DESCRIPTION :
  --   Prompts the user for number of rows and columns
  --   and then shows a matrix of random complex double doubles.

  procedure Main;

  -- DESCRIPTION :
  --   Displays a menu and prompts for a test.

end Test_DoblDobl_Matrices;
