with Test_DecaDobl_Singular_Values;

procedure ts_dasvd is

-- DESCRIPTION :
--   Calls the main interactive test on the Singular Value Decomposition
--   in deca double precision.

begin
  Test_DecaDobl_Singular_Values.Main;
end ts_dasvd;
