with Multprec_Natural_Coefficients;      use Multprec_Natural_Coefficients;

package Multprec_Natural_Coefficients_io is

-- DESCRIPTION :
--   Very basic output useful for testing.

  procedure put ( n : Array_of_Naturals );

  -- DESCRIPTION :
  --   Writes the array to screen.

end Multprec_Natural_Coefficients_io;
