with Test_Multprec_Singular_Values;

procedure ts_mpsvd is

-- DESCRIPTION :
--   Calls the main interactive test on the Singular Value Decomposition
--   in arbitrary multiprecision.

begin
  Test_Multprec_Singular_Values.Main;
end ts_mpsvd;
