with Generic_Polynomials;
with Double_Double_Ring;

package Double_Double_Polynomials is 
  new Generic_Polynomials(Double_Double_Ring);

-- DESCRIPTION :
--   Defines polynomials over the ring of double double numbers.
