with Test_Floating_Numbers;

procedure ts_fltnum is

-- DESCRIPTION :
--   Calls the main interactive tests o multiprecision floating-point numbers.

begin
  Test_Floating_Numbers.Main;
end ts_fltnum;
