with Test_Standard_Singular_Values;

procedure ts_dsvd is

-- DESCRIPTION :
--   Calls the main interactive test on the Singular Value Decomposition
--   in standard double precision.

begin
  Test_Standard_Singular_Values.Main;
end ts_dsvd;
