with Test_DecaDobl_Vector_Norms;

procedure ts_danorm is

-- DESCRIPTION :
--   Calls the main interactive test on vector norms
--   in deca double precision.

begin
  Test_DecaDobl_Vector_Norms.Main;
end ts_danorm;
