with Test_Multprec_QRLS_Solvers;

procedure ts_mpqrd is

-- DESCRIPTION :
--   Calls the main interactive test on the QR Decomposition and
--   the least squares solver in arbitrary multiprecision.

begin
  Test_Multprec_QRLS_Solvers.Main;
end ts_mpqrd;
