with Test_Standard_QRLS_Solvers;

procedure ts_dqrd is

-- DESCRIPTION :
--   Calls the main interactive test on the QR Decomposition and
--   the least squares solver in standard double precision.

begin
  Test_Standard_QRLS_Solvers.Main;
end ts_dqrd;
