package Test_Solutions_io is

-- DESCRIPTION :
--   Provides simple tests on the input and output of solution lists.

  procedure Double_Read_Write;

  -- Reads a list of solutions in double precision
  -- from standard input and writes it back to screen.

  procedure DoblDobl_Read_Write;

  -- Reads a list of solutions in double double precision
  -- from standard input and writes it back to screen.

  procedure TripDobl_Read_Write;

  -- Reads a list of solutions in triple double precision
  -- from standard input and writes it back to screen.

  procedure QuadDobl_Read_Write;

  -- Reads a list of solutions in quad double precision
  -- from standard input and writes it back to screen.

  procedure PentDobl_Read_Write;

  -- Reads a list of solutions in penta double precision
  -- from standard input and writes it back to screen.

  procedure OctoDobl_Read_Write;

  -- Reads a list of solutions in octo double precision
  -- from standard input and writes it back to screen.

  procedure DecaDobl_Read_Write;

  -- Reads a list of solutions in deca double precision
  -- from standard input and writes it back to screen.

  procedure HexaDobl_Read_Write;

  -- Reads a list of solutions in hexa double precision
  -- from standard input and writes it back to screen.

  procedure Main;

  -- DESCRIPTION :
  --   Displays a menu and prompts for a choice of test.

end Test_Solutions_io;
