with Test_OctoDobl_Linear_Solvers;

procedure ts_odlu is

-- DESCRIPTION :
--   Calls the main interactive solver of linear systems
--   in octo double precision.

begin
  Test_OctoDobl_Linear_Solvers.Main;
end ts_odlu;
