with Generic_Polynomials;
with Deca_Double_Ring;

package Deca_Double_Polynomials is 
  new Generic_Polynomials(Deca_Double_Ring);

-- DESCRIPTION :
--   Defines polynomials over the ring of deca double numbers.
