img
{
   max-width: 100%;
   height: auto;
   width: auto\9; /* ie8 */
}
.center1
{
   width: auto;
   min-height: 350px;
   margin-left: auto;
   margin-right: auto;
}
.signin-box
{
   padding: 20px 20px 15px;
   background: #f1f1f1;
}
input[type="text"],input[type="password"],input[type="email"]
{
   width: 100%;
   max-width: 230px;
   font-size: 15px;
}
.reg_form
{
   width: 100px;
}
.g-button 
{
   display: inline-block;
   min-width: 46px;
   text-align: center;
   color: #444;
   font-size: 11px;
   font-weight: bold;
   height: 27px;
   padding: 0 8px;
   line-height: 27px;
   -webkit-border-radius: 2px;
   -moz-border-radius: 2px;
   border-radius: 2px;
   -webkit-transition: all 0.218s;
   -moz-transition: all 0.218s;
   -ms-transition: all 0.218s;
   -o-transition: all 0.218s;
   transition: all 0.218s;
   border: 0px solid;
   background-color: #f5f5f5;
   background-image: -webkit-gradient(linear,left top,left bottom,from(#f5f5f5),to(#f1f1f1));
   background-image: -webkit-linear-gradient(top,#f5f5f5,#f1f1f1);
   background-image: -moz-linear-gradient(top,#f5f5f5,#f1f1f1);
   background-image: -ms-linear-gradient(top,#f5f5f5,#f1f1f1);
   background-image: -o-linear-gradient(top,#f5f5f5,#f1f1f1);
   background-image: linear-gradient(top,#f5f5f5,#f1f1f1);
   -webkit-user-select: none;
   -moz-user-select: none;
   user-select: none;
   cursor: default;
}
.g-button-submit 
{
   color: #fff;
   text-shadow: 0 1px rgba(0,0,0,0.1);
   background-color: #4d90fe;
   background-image: -webkit-gradient(linear,left top,left bottom,from(#4d90fe),to(#4787ed));
   background-image: -webkit-linear-gradient(top,#4d90fe,#4787ed);
   background-image: -moz-linear-gradient(top,#4d90fe,#4787ed);
   background-image: -ms-linear-gradient(top,#4d90fe,#4787ed);
   background-image: -o-linear-gradient(top,#4d90fe,#4787ed);
   background-image: linear-gradient(top,#4d90fe,#4787ed);
}
.g-button-submit:hover 
{
   color: #fff;
   text-shadow: 0 1px rgba(0,0,0,0.3);
   background-color: #357ae8;
   background-image: -webkit-gradient(linear,left top,left bottom,from(#4d90fe),to(#357ae8));
   background-image: -webkit-linear-gradient(top,#4d90fe,#357ae8);
   background-image: -moz-linear-gradient(top,#4d90fe,#357ae8);
   background-image: -ms-linear-gradient(top,#4d90fe,#357ae8);
   background-image: -o-linear-gradient(top,#4d90fe,#357ae8);
   background-image: linear-gradient(top,#4d90fe,#357ae8);
}
.g-button-submit:active 
{
   background-color: #357ae8;
   background-image: -webkit-gradient(linear,left top,left bottom,from(#4d90fe),to(#357ae8));
   background-image: -webkit-linear-gradient(top,#4d90fe,#357ae8);
   background-image: -moz-linear-gradient(top,#4d90fe,#357ae8);
   background-image: -ms-linear-gradient(top,#4d90fe,#357ae8);
   background-image: -o-linear-gradient(top,#4d90fe,#357ae8);
   background-image: linear-gradient(top,#4d90fe,#357ae8);
   -webkit-box-shadow: inset 0 1px 2px rgb	a(0,0,0,0.3);
   -moz-box-shadow: inset 0 1px 2px rgba(0,0,0,0.3);
   box-shadow: inset 0 1px 2px rgba(0,0,0,0.3);
}
