with Test_DoblDobl_Polynomials;

procedure ts_ddpol is

-- DESCRIPTION :
--   Calls the main test on polynomials with double double coefficients.

begin
  Test_DoblDobl_Polynomials.Main;
end ts_ddpol;
