with Test_Multprec_DoblDobl_Casts;

procedure ts_mpddcv is

-- DESCRIPTION :
--   Calls the main interactive test on type casts between
--   multiprecision and double double precision numbers.

begin
  Test_Multprec_DoblDobl_Casts.Main;
end ts_mpddcv;
