with DoblDobl_Complex_Series_Ring;
with Generic_Vectors;

package DoblDobl_Complex_Series_Vectors is 
  new Generic_Vectors(DoblDobl_Complex_Series_Ring);

-- DESCRIPTION :
--   Defines vectors over the ring of dobldobl dense series.
