with Standard_Complex_Ring;
with Generic_Vectors;

package Standard_Complex_Vectors is new Generic_Vectors(Standard_Complex_Ring);

-- DESCRIPTION :
--   Defines vectors over the ring of standard complex numbers.
