with Generic_Polynomials;
with DoblDobl_Complex_Ring;

package DoblDobl_Complex_Polynomials is 
  new Generic_Polynomials(DoblDobl_Complex_Ring);

-- DESCRIPTION :
--   Defines polynomials over the ring of double double complex numbers.
