with Standard_Integer_Ring;
with Generic_Vectors;

package Standard_Integer_Vectors is new Generic_Vectors(Standard_Integer_Ring);

-- DESCRIPTION :
--   Defines vectors over the ring of standard integer numbers.
