with Test_QuadDobl_Functions;

procedure ts_qdfun is

-- DESCRIPTION :
--   Calls the main interactive test on functions in quad double precision.

begin
  Test_QuadDobl_Functions.Main;
end ts_qdfun;
