with Test_QuadDobl_Singular_Values;

procedure ts_qdsvd is

-- DESCRIPTION :
--   Calls the main interactive test on the Singular Value Decomposition
--   in quad double precision.

begin
  Test_QuadDobl_Singular_Values.Main;
end ts_qdsvd;
