with Test_Integer_Linear_Solvers;

procedure ts_intlina is

-- DESCRIPTION :
--   Calls the main interactive test on the integer linear solvers.

begin
  Test_Integer_Linear_Solvers.Main;
end ts_intlina;
