with Test_DecaDobl_QRLS_Solvers;

procedure ts_daqrd is

-- DESCRIPTION :
--   Calls the main interactive test on the QR Decomposition and
--   the least squares solver in deca double precision.

begin
  Test_DecaDobl_QRLS_Solvers.Main;
end ts_daqrd;
