with Drivers_for_Condition_Tables;       use Drivers_for_Condition_Tables; 

procedure ts_contab is

-- DESCRIPTION :
--   Test on the calculation of condition tables,
--   especially for huge lists of solutions.

begin
  Interactive_Driver_to_Scan_Solution_Lists;
end ts_contab;
