with QuadDobl_Dense_Series_Ring;
with Generic_Vectors;

package QuadDobl_Dense_Series_Vectors is 
  new Generic_Vectors(QuadDobl_Dense_Series_Ring);

-- DESCRIPTION :
--   Defines vectors over the ring of quad double dense series.
