with Generic_Polynomials;
with QuadDobl_Complex_Ring;

package QuadDobl_Complex_Polynomials is 
  new Generic_Polynomials(QuadDobl_Complex_Ring);

-- DESCRIPTION :
--   Defines polynomials over the ring of quad double complex numbers.
