with Test_Vector_Strings;

procedure ts_strvec is

-- DESCRIPTION :
--   Calls the main interactive test on writing vectors to strings
--   and parsing strings into vectors.

begin
  Test_Vector_Strings.Main;
end ts_strvec;
