with Standard_Integer64_Ring;
with Generic_Vectors;

package Standard_Integer64_Vectors is 
  new Generic_Vectors(Standard_Integer64_Ring);

-- DESCRIPTION :
--   Defines vectors over the ring of standard integer numbers,
--   of type long long integer.
