package Test_Theta_Algorithm is

-- DESCRIPTION :
--   Reproduces the results of the 1971 paper of Brezinski
--   on the theta algorithm, using examples 2 and 3 of the paper.

  procedure Main;

  -- DESCRIPTION :
  --   Prompts to select a test.

end Test_Theta_Algorithm;
