// cuda_set.h, created by yxc on Feb 3, 2015 with edits by jv

#ifndef __CUDA_SET__H_
#define __CUDA_SET__H_

#include <iostream>

void cuda_set();
// sets the device with an error check

#endif /* __CUDA_SET__H_ */
