with Test_TripDobl_Vector_Norms;

procedure ts_tdnorm is

-- DESCRIPTION :
--   Calls the main interactive test on vector norms
--   in triple double precision.

begin
  Test_TripDobl_Vector_Norms.Main;
end ts_tdnorm;
