with text_io;                               use text_io;

package Matrix_Homotopies_io is

-- DESCRIPTION :
--   Provides a simple output routine for writing the matrix homotopies.

  procedure Write;
  procedure Write ( file : in file_type );

end Matrix_Homotopies_io;
