with Generic_Polynomials;
with Penta_Double_Ring;

package Penta_Double_Polynomials is 
  new Generic_Polynomials(Penta_Double_Ring);

-- DESCRIPTION :
--   Defines polynomials over the ring of penta double numbers.
