with Test_mtSeries_Linearization;

procedure ts_mtserlin is

-- DESCRIPTION :
--   Calls the main test on the linearization of solving linear systems 
--   of truncated series with multitasking.

begin
  Test_mtSeries_Linearization.Main;
end ts_mtserlin;
