package Test_OctoDobl_Random_Systems is

-- DESCRIPTION :
--   Tests random polynomials in octo double precision.

  procedure Main;

  -- DESCRIPTION :
  --   Prompts for the setup and then generates a random system.

end Test_OctoDobl_Random_Systems;
