with text_io;                            use text_io;

procedure Write_Seed_Number ( file : in file_type );

-- DESCRIPTION :
--   This procedure defines the layout of the message to write the
--   seed used to generate the random numbers in a run of phc.
