with Test_Floating_Vectors;

procedure ts_fltvec is

-- DESCRIPTION :
--   Calls the interactive main test on vectors of standard double
--   and multiprecision floating-point numbers.

begin
  Test_Floating_Vectors.Main;
end ts_fltvec;
