with Test_DoblDobl_Functions;

procedure ts_ddfun is

-- DESCRIPTION :
--   Calls the main interactive test on functions in double double precision.

begin
  Test_DoblDobl_Functions.Main;
end ts_ddfun;
